"""
This Source is the Main part of QiRub Project

QiRub is a Project for Rubika Client Handler
QiRub includes ( 'httpx', 'pycryptodome', 'fake_useragent' )

In fact when you import the ClientMessenger class or
    run the source what called ClientMessenger class, its automatically set a
        random fake user agent to have a better connection, you can off/on that on ClientMessenger class on the
            `UseFakeUserAgent` Parameter with boolean types, also you can use Proxy, on the `Proxy` Parameter to set your proxies and have a better connection
        

Make Sure you use the latest version of QiRub, for
More info you can visit our github, also you can access
To new News, Updates, Bug Fixes, ... with github

Github: https://github.com/Rubier-Project/QiRub

"""

from .network import QiNetwork
from .updater import QiUpdater
from .DataParse import Parse
import time

__version__ = "1.0.0"

class ClientMessenger(object):
    def __init__(self, AuthToken: str, PrivateKey: str, UseFakeUserAgent: bool = True, Proxy = None):
        self.authtoken = AuthToken
        self.privatekey = PrivateKey.replace("-----BEGIN PRIVATE KEY-----\n", "").replace("\n-----END PRIVATE KEY-----", "")

        self.ufa = UseFakeUserAgent
        self.proxy = Proxy
        
        self.network = QiNetwork(self.authtoken, self.privatekey, Proxy)
    
    @property
    def accountInfo(self):
        return self.network.option({}, "getUserInfo", self.ufa)
    
    def handlePhoneNumber(self, phone: str):
        p = ""
        if phone.startswith("0"):
            p = phone[1:]
        elif phone.startswith("98"):
            p = phone[2:]
        elif phone.startswith("+98"):
            p = phone[3:]
        else:
            p = phone
        
        return p
    
    def guessGuid(self, guid: str):
        if guid.startswith("c0"):
            return "Channel"
        elif guid.startswith("g0"):
            return "Group"
        elif guid.startswith("u0"):
            return "User"
        elif guid.startswith("b0"):
            return "Bot"
        elif guid.startswith("s0"):
            return "Service"
        
    def endpointHash(self, link: str):
        if "/" in link:
            return link.split("/")[-1]
        else:
            return link

    def getMe(self):
        return self.network.option({}, "getUserInfo", self.ufa)

    def getChatsUpdates(self):
        return self.network.option({"state": time.time() - 150}, "getChatsUpdates", self.ufa)
    
    def getChats(self):
        return self.network.option({"start_id": None}, "getChats", self.ufa)

    def onMessage(self):
        yield QiUpdater(self.authtoken, self.privatekey, self.getChatsUpdates(), self.ufa, self.proxy)

    def onChatMessage(self, chat_object_guid: str):
        try:
            datas = self.getChatsUpdates()
            if datas['data']['chats'][0]['object_guid'] == chat_object_guid:
                return datas['data']['chats'][0]
            else:
                return {}
        except Exception as ERROR:
            return {"error": True, "base": str(ERROR)}
    
    @property
    def send_code_types(self):
        return ("SMS", "Internal")

    def sendCode(self, phone_number: str, pass_key: str, send_type: str = "SMS", parsePhoneNumber: bool = True):
        if not send_type in self.send_code_types:
            raise ValueError(f"Send type does not available, use send_code_types property to see more of that")
        return self.network.option({"phone_number": self.handlePhoneNumber(phone_number), "pass_key": pass_key,
                                    "send_type": send_type}, "sendCode", self.ufa) if parsePhoneNumber else self.network.option({"phone_number": phone_number, "pass_key": pass_key,
                                    "send_type": send_type}, "sendCode", self.ufa)
    
    def signIn(self, phone_number: str, phone_code: str, phone_code_hash: str, public_key: str, parsePhoneNumber: bool = True):
        return self.network.option({"phone_number": phone_number, "phone_code": phone_code,
                                    "phone_code_hash": phone_code_hash, "public_key": public_key}, "signIn", self.ufa) if parsePhoneNumber else self.network.option({"phone_number": self.handlePhoneNumber(phone_number), "phone_code": phone_code,
                                    "phone_code_hash": phone_code_hash, "public_key": public_key}, "signIn", self.ufa)

    def addChannel(self, title: str, description: str = None, member_guids: list = None):
        return self.network.option({"title": title, "description": description,
                                    "member_guids": member_guids}, "addChannel", self.ufa) if type(member_guids) == list else self.network.option({"title": title, "description": description,
                                    "member_guids": [member_guids]}, "addChannel", self.ufa)
    
    def addChannelMembers(self, object_guid: str, member_guids: list = None):
        return self.network.option({"channel_guid": object_guid, "member_guids": member_guids}, "addChannelMembers", self.ufa) if type(member_guids) == list else \
               self.network.option({"channel_guid": object_guid, "member_guids": [member_guids]}, "addChannelMembers", self.ufa)
    
    @property
    def ban_channel_member_actions(self):
        return ("Set", "Unset")

    def banChannelMember(self, object_guid: str, member_guid: str):
        return self.network.option({"channel_guid": object_guid, "member_guid": member_guid,
                                    "action": "Set"}, "banChannelMember", self.ufa)
    
    def banChannelMembers(self, object_guid: str, member_guids: list):

        if type(member_guids) != list and type(member_guids) == str:
            member_guids = [member_guids]

        dbs = {}

        for _ in member_guids:
            dbs[_] = self.banChannelMember(object_guid=object_guid, member_guid=_)
        
        return dbs

    def channelPreviewByJoinLink(self, link: str, use_endpoint_hash: bool = True):
        return self.network.option({"hash_link": self.endpointHash(link)},
                                   "channelPreviewByJoinLink", self.ufa) if use_endpoint_hash else self.network.option({"hash_link": link},
                                   "channelPreviewByJoinLink", self.ufa)
    
    def checkChannelUsername(self, username: str, replace_hashtag: bool = True):
        return self.network.option({"username": username.replace("@", "")},
                                   "checkChannelUsername", self.ufa) if replace_hashtag else self.network.option({"username": username},
                                   "checkChannelUsername", self.ufa)
    
    def checkChannelUsernames(self, usernames: list, replace_hashtag: bool = True):
        if not type(usernames) == list:
            raise ValueError("`usernames` parameter in checkChannelUsernames is not list")
        
        dbs = {}

        for username in usernames:
            dbs[username] = self.checkChannelUsername(username=username, replace_hashtag=replace_hashtag)

        return dbs
    
    def createChannelVoiceChat(self, channel_guid: str):
        return self.network.option({"channel_guid": channel_guid},
                                   "createChannelVoiceChat", self.ufa)
    
    def deleteNoAccessGroupChat(self, group_guid: str):
        return self.network.option({"group_guid": group_guid},
                                   "deleteNoAccessGroupChat", self.ufa)
    
    def discardChannelVoiceChat(self, channel_guid: str, voice_chat_id: str):
        return self.network.option({"channel_guid": channel_guid, 
                                    "voice_chat_id": voice_chat_id}, "discardChannelVoiceChat", self.ufa)
    
    @property
    def chat_history_for_new_members_list(self):
        return ('Hidden', 'Visible')

    def editChannelInfo(self,
                channel_guid: str,
                title: str = None,
                description: str = None,
                channel_type: str = None,
                sign_messages: str = None,
                chat_reaction_setting: dict = None,
                chat_history_for_new_members: str = "Hidden"):
        
        updatedParameters = []
        inp = {
            "channel_guid": channel_guid
        }

        if title is not None:
            inp['title'] = title
            updatedParameters.append('title')

        if description is not None:
            inp['description'] = description
            updatedParameters.append('description')

        if channel_type is not None:
            inp['channel_type'] = channel_type
            updatedParameters.append('channel_type')

        if sign_messages is not None:
            inp['sign_messages'] = sign_messages
            updatedParameters.append('sign_messages')

        if chat_reaction_setting is not None:
            inp['chat_reaction_setting'] = chat_reaction_setting
            updatedParameters.append('chat_reaction_setting')

        if chat_history_for_new_members is not None:
            if chat_history_for_new_members not in self.chat_history_for_new_members_list:
                raise ValueError('`chat_history_for_new_members` parameter in editChannelInfo is not available, to see more options use `chat_history_for_new_members_list` property.')

            inp['chat_history_for_new_members'] = chat_history_for_new_members
            updatedParameters.append('chat_history_for_new_members')

        inp['updated_parameters'] = updatedParameters

        return self.network.option(inp, "editChannelInfo", self.ufa)
    
    def getBannedGroupMembers(self, group_guid: str, start_id: str = None):
        return self.network.option({
            "group_guid": group_guid,
            "start_id": start_id
        }, "getBannedGroupMembers", self.ufa)
    
    def getChannelAdminAccessList(self, channel_guid: str, admin_user_guid: str):
        return self.network.option({
            "channel_guid": channel_guid,
            "member_guid": admin_user_guid
        }, "getChannelAdminAccessList", self.ufa)
    
    def getChannelAdminsAccessList(self, channel_guid: str, admins_user_guids: list):
        if not type(admins_user_guids) == list:
            raise ValueError("`admins_user_guids` parameter in getChannelAdminsAccessList is not list")
        
        dbs = {}

        for guid in admins_user_guids:
            dbs[guid] = self.getChannelAdminAccessList(channel_guid=channel_guid, admin_user_guid=guid)

        return dbs
    
    def getChannelAdminMembers(self, channel_guid: str, start_id: str = None):
        return self.network.option({
            "channel_guid": channel_guid,
            "start_id": start_id
        }, "getChannelAdminMembers", self.ufa)
    
    def getChannelAllMembers(self, channel_guid: str, search_text: str = '', start_id: str = None):
        return self.network.option({
            "channel_guid": channel_guid,
            "start_id": start_id
        }, "getChannelAllMembers", self.ufa)
    
    def getChannelInfo(self, channel_guid: str):
        return self.network.option({"channel_guid": channel_guid}, "getChannelInfo", self.ufa)
    
    def getChannelLink(self, channel_guid: str):
        return self.network.option({"channel_guid": channel_guid}, "getChannelLink", self.ufa)
    
    def getGroupDefaultAccess(self, group_guid: str):
        return self.network.option({"group_guid": group_guid}, "getGroupDefaultAccess", self.ufa)
    
    def getGroupMentionList(self, group_guid: str, search_mention: str = None):
        return self.network.option({"group_guid": group_guid, "search_mention": search_mention}, "getGroupMentionList", self.ufa)
    
    def getGroupVoiceChatUpdates(self, group_guid: str, voice_chat_id: str  ):
        return self.network.option({"group_guid": group_guid, "voice_chat_id": voice_chat_id, "state": round(time.time()) - 150}, "getGroupVoiceChatUpdates", self.ufa)
    
    @property
    def join_channel_actions(self):
        return (
            "Join",
            "Remove",
            "Archive"
        )
    
    def joinChannelAction(self, channel_guid: str, action: str):
        if not action in self.join_channel_actions:
            raise ValueError("`action` parameter in joinChannelAction does not available, to see more actions print `join_channel_actions` property")
        
        return self.network.option({"channel_guid": channel_guid, "action": action}, "joinChannelAction", self.ufa)
    
    def joinChannelByLink(self, link: str, use_endpoint_hash: bool = True):
        return self.network.option({"hash_link": self.endpointHash(link)},
                                   "joinChannelByLink", self.ufa) if use_endpoint_hash else self.network.option({"hash_link": self.endpointHash(link)},
                                   "joinChannelByLink", self.ufa)
    
    def joinGroup(self, link: str, use_endpoint_hash: bool = True):
        return self.network.option({"hash_link": self.endpointHash(link)},
                                   "joinGroup", self.ufa) if use_endpoint_hash else self.network.option({"hash_link": self.endpointHash(link)},
                                   "joinGroup", self.ufa)
    
    def leaveGroup(self, group_guid: str):
        return self.network.option({"group_guid": group_guid}, "leaveGroup", self.ufa)
    
    def leaveGroupVoiceChat(self, group_guid: str, voice_chat_id: str):
        return self.network.option({"group_guid": group_guid, "voice_chat_id": voice_chat_id}, "leaveGroupVoiceChat", self.ufa)
    
    def removeChannel(self, channel_guid: str):
        return self.network.option({"channel_guid": channel_guid}, "removeChannel", self.ufa)
    
    def seenChannelMessages(self, channel_guid: str, min_id: int, max_id: int):
        return self.network.option({"channel_guid": channel_guid, "min_id": min_id, "max_id": max_id}, "seenChannelMessages", self.ufa)
    
    def setChannelLink(self, channel_guid: str):
        return self.network.option({"channel_guid": channel_guid}, "setChannelLink", self.ufa)
    
    def changeChannelLink(self, channel_guid: str):
        return self.setChannelLink(channel_guid=channel_guid)
    
    def setChannelVoiceChatSetting(self, channel_guid: str, voice_chat_id: str, title: str = None):
        inp = {
            "channel_guid": channel_guid,
            "voice_chat_id": voice_chat_id
        }

        updatedParameters = []

        if title is not None:
            inp['title'] = title
            updatedParameters.append("title")

        inp['updated_parameters'] = updatedParameters

        return self.network.option(inp, "setChannelVoiceChatSetting", self.ufa)
    
    def changeChannelVoiceChatSetting(self, channel_guid: str, voice_chat_id: str, title: str = None):
        return self.setChannelVoiceChatSetting(channel_guid=channel_guid, voice_chat_id=voice_chat_id, title=title)
    
    @property
    def group_admins_actions(self):
        return (
            "SetAdmin",
            "UnsetAdmin"
        )
    
    def setGroupAdmin(self, channel_guid: str, member_guid: str, action: str = "SetAdmin", access_list: list = []):
        if not action in self.group_admins_actions:
            raise ValueError("`action` parameter in ( setGroupAdmin / addGroupAdmin ) functions, is not available, to see more actions print `group_admins_actions` property")
        
        if type(access_list) != list and type(access_list) == str:
            access_list = [access_list]

        return self.network.option({"channel_guid": channel_guid, "member_guid": member_guid,
                                    "action": action, "access_list": access_list}, "setGroupAdmin", self.ufa)
    
    def addGroupAdmin(self, channel_guid: str, member_guid: str, action: str = "SetAdmin", access_list: list = []):
        return self.setGroupAdmin(channel_guid=channel_guid, member_guid=member_guid,
                                  action=action, access_list=access_list)
    
    def updateChannelUsername(self, channel_guid: str, username: str, replace_hashtag: bool = True):
        return self.network.option({"channel_guid": channel_guid,
                                    "username": username.replace("@", "")}, "updateChannelUsername", self.ufa) if replace_hashtag else self.network.option({"channel_guid": channel_guid,
                                    "username": username.replace("@", "")}, "updateChannelUsername", self.ufa)



"""
This Source is part of QiRub Project

Make Sure you use the latest version of QiRub, for
More info you can visit our github, also you can access
To new News, Updates, Bug Fixes, ... with github

Github: https://github.com/Rubier-Project/QiRub

"""

import httpx
import random
import fake_useragent
import json

from .crypto import encryption

class ProxyType(object):
    ...

class QiStream(object):
    def choiceObject(items: list = []):
        return random.choice(items)
    
    def randomIntSteram() -> int:
        return random.randint(100000, 999999999)
    
    def randomStrStream() -> str:
        return str(random.randint(-99999999, 99999999))
    
    def deviceHashGenerator() -> str:
        return "".join(random.choices("0123456789", k=26))

class QiNetwork(object):
    def __init__(self, AuthToken: str, PrivateKey: str, Proxy: ProxyType = None):
        self.auth = AuthToken
        self.key = PrivateKey
        self.proxy = Proxy
        self.agent = fake_useragent.UserAgent().random

        self.enc = encryption(self.auth, self.key)
        
        self.newAuth = encryption.change(self.auth)
        
        self.client = {
            "app_name": "Main",
            "app_version": "3.5.7",
            "lang_code": "fa",
            "package": "app.rbmain.a",
            "temp_code": "27",
            "platform": "Android"
        }
        
        self.apis = [
            "https://messengerg2c2.iranlms.ir",
            "https://messengerg2c3.iranlms.ir"
        ]

        self.selectedApi = QiStream.choiceObject(self.apis)
    
    def option(self,
               input_data: dict,
               method: str,
               use_fake_useragent: bool = True
               ):
        
        data = json.dumps({
            "input": input_data,
            "method": method,
            "client": self.client
        })

        encs = self.enc.encrypt(data)
        sig = self.enc.Sign(encs)

        notData = json.dumps({
            "api_version": "6",
            "auth": self.newAuth,
            "sign": sig,
            "data_enc": encs
        })

        heads = {"User-Agent": self.agent, "Referer": "https://rubika.ir"} if use_fake_useragent else {"Referer": "https://rubika.ir"}

        net = httpx.Client(proxy=self.proxy)

        try:
            data = json.loads(self.enc.decrypt(json.loads(net.post(self.selectedApi, data=notData, headers=heads).text)['data_enc']))
            return data
        except Exception as ERROR_QI:
            return str(ERROR_QI)



"""
This Source is part of QiRub Project

Make Sure you use the latest version of QiRub, for
More info you can visit our github, also you can access
To new News, Updates, Bug Fixes, ... with github

Github: https://github.com/Rubier-Project/QiRub

"""

import base64,crypto.Util.Padding,crypto.Hash
from crypto.Cipher import AES
from crypto.Signature import pkcs1_15
from crypto.PublicKey import RSA 

class encryption:
  def __init__(self, auth:str, private_key:str=None):
    self.auth = auth
    self.key = bytearray(self.secret(auth), "UTF-8")
    self.iv = bytearray.fromhex('00000000000000000000000000000000')
    self.keypair = RSA.import_key("-----BEGIN PRIVATE KEY-----\n"+private_key+"\n-----END PRIVATE KEY-----") if private_key else None

  def change(auth_enc):
        return ''.join([chr(((32 - (ord(s) - 97)) % 26) + 97) if s.islower() else chr(((29 - (ord(s) - 65)) % 26) + 65) if s.isupper() else chr(((13 - (ord(s) - 48)) % 10) + 48) if s.isdigit() else s for s in auth_enc])
  def secret(self, e):
        return ''.join([chr((ord(char) - ord('0') + 5) % 10 + ord('0')) if char.isdigit() else chr((ord(char) - ord('a') + 9) % 26 + ord('a')) for char in (e[16:24] + e[0:8] + e[24:32] + e[8:16])])
  def encrypt(self, text):
        return base64.b64encode(AES.new(self.key, AES.MODE_CBC, self.iv).encrypt(Cryptodome.Util.Padding.pad(text.encode('UTF-8'), AES.block_size))).decode('UTF-8')
  def decrypt(self, text):
        return Cryptodome.Util.Padding.unpad(AES.new(self.key, AES.MODE_CBC, self.iv).decrypt(base64.urlsafe_b64decode(text.encode('UTF-8'))), AES.block_size).decode('UTF-8')
  def Sign(self, data_enc:str):
        return base64.b64encode(pkcs1_15.new(self.keypair).sign(Cryptodome.Hash.SHA256.new(data_enc.encode("utf-8")))).decode("utf-8")
  def decryptRsaOaep(private:str,data_enc:str):
        return Cryptodome.Cipher.PKCS1_OAEP.new(RSA.import_key(private.encode("utf-8"))).decrypt(base64.b64decode(data_enc)).decode("utf-8")
  def rsaKey():
       Grt = RSA.generate(1024)
       return encryption.change(base64.b64encode(Grt.publickey().export_key()).decode("utf-8")),Grt.export_key().decode("utf-8")
  def auth_decrypt(key):
       return ''.join([chr(((13 - (ord(s) - 48)) % 10) + 48) if s.isdigit() else chr(((32 - (ord(s) - 97)) % 26) + 97) if s.islower() else chr(((29 - (ord(s) - 65)) % 26) + 65) if s.isupper() else s for s in key])



import os
import json
import re
import rich

os.system("")

class Parse(object):
    def __init__(self) -> None:
        pass

    def TupleParse(self, data: tuple):
        d = "\033[00m( "
        num = 0

        for item in data:
            num += 1

            if type(item) == str:
                if len(data) - num == 0:
                    d += f"\033[32m'{item}'\033[00m "
                else:
                    d += f"\033[32m'{item}'\033[00m, "

            if type(item) == int:
                if len(data) - num == 0:
                    d += f"\033[93m{item}\033[00m "
                else:
                    d += f"\033[93m{item}\033[00m, "

            if type(item) == bool:
                if len(data) - num == 0:
                    if item == True:
                        d += f"\033[94mtrue\033[00m "
                    else:
                        d += f"\033[91mfalse\033[00m "
                
                else:
                    if item == True:
                        d += f"\033[94mtrue\033[00m, "
                    else:
                        d += f"\033[91mfalse\033[00m, "

            if type(item) == tuple:
                if len(data) - num == 0:
                    d += f"\033[00m{self.TupleParse(item)}\033[00m "
                else:
                    d += f"\033[00m{self.TupleParse(item)}\033[00m "

            if type(item) == dict:
                if len(data) - num == 0:
                    d += f"\033[00m{self.DictParse(item)}\033[00m "
                else:
                    d += f"\033[00m{self.DictParse(item)}\033[00m "

            if type(item) == list:
                if len(data) - num == 0:
                    d += f"\033[00m{self.ListParse(item)}\033[00m "
                else:
                    d += f"\033[00m{self.ListParse(item)}\033[00m "

            if type(item) == set:
                if len(data) - num == 0:
                    d += f"\033[00m{self.SetParse(item)}\033[00m "
                else:
                    d += f"\033[00m{self.SetParse(item)}\033[00m "

        d += "\033[00m)"

        return d

    def SetParse(self, data: set):
        d = "\033[00m{ "
        num = 0

        for item in data:
            num += 1
            
            if type(item) == str:
                if len(data) - num == 0:
                    d += f"\033[32m'{item}'\033[00m "
                else:
                    d += f"\033[32m'{item}'\033[00m, "

            if type(item) == int:
                if len(data) - num == 0:
                    d += f"\033[93m{item}\033[00m "
                else:
                    d += f"\033[93m{item}\033[00m, "

            if type(item) == bool:
                if len(data) - num == 0:
                    if item == True:
                        d += f"\033[94mtrue\033[00m "
                    else:
                        d += f"\033[91mfalse\033[00m "

                else:
                    if item == True:
                        d += f"\033[94mtrue\033[00m, "
                    else:
                        d += f"\033[91mfalse\033[00m, "

        d += " \033[00m}"

        return d

    def ListParse(self, data: list):
        d = "\033[00m[ "
        num = 0

        for item in data:
            num += 1

            if type(item) == str:
                if len(data) - num == 0:
                    d += f"\033[32m'{item}'\033[00m "
                else:
                    d += f"\033[32m'{item}'\033[00m, "
            
            if type(item) == int:
                if len(data) - num == 0:
                    d += f"\033[93m{item}\033[00m "
                else:
                    d += f"\033[93m{item}\033[00m, "

            if type(item) == bool:
                if len(data) - num == 0:
                    if item == True:
                        d += f"\033[94mtrue\033[00m "
                    else:
                        d += f"\033[91mfalse\033[00m "

                else:
                    if item == True:
                        d += f"\033[94mtrue\033[00m, "
                    else:
                        d += f"\033[91mfalse\033[00m, "

            if type(item) == list:
                if len(data) - num == 0:
                    d += f"\033[93m{self.ListParse(item)}\033[00m "
                else:
                    d += f"\033[93m{self.ListParse(item)}\033[00m, "

            if type(item) == dict:
                if len(data) - num == 0:
                    d += f"\033[93m{self.DictParse(item)}\033[00m "
                else:
                    d += f"\033[93m{self.DictParse(item)}\033[00m, "

            if type(item) == tuple:
                if len(data) - num == 0:
                    d += f"\033[00m{self.TupleParse(item)}\033[00m "
                else:
                    d += f"\033[00m{self.TupleParse(item)}\033[00m "

            if type(item) == set:
                if len(data) - num == 0:
                    d += f"\033[00m{self.SetParse(item)}\033[00m "
                else:
                    d += f"\033[00m{self.SetParse(item)}\033[00m "

        d += "\033[00m]"

        return d

    def DictParse(self, data: dict):
        d = "\033[00m{ "
        num = 0

        for key, val in data.items():
            num += 1

            if type(key) == str and type(val) == str:
                if len(data.keys()) - num == 0:
                    d += f"\033[00m{key}: \033[32m'{val}'\033[00m "
                else:
                    d += f"\033[00m{key}: \033[32m'{val}'\033[00m, "

            if type(key) == str and type(val) == int:
                if len(data.keys()) - num == 0:
                    d += f"\033[00m{key}: \033[93m{val}\033[00m "
                else:
                    d += f"\033[00m{key}: \033[93m{val}\033[00m, "

            if type(key) == str and type(val) == bool:
                if len(data.keys()) - num == 0:
                    if str(val) == "True":
                        d += f"\033[00m{key}: \033[94mtrue\033[00m "
                    else:
                        d += f"\033[00m{key}: \033[91mfalse\033[00m "
                else:
                    if str(val) == "True":
                        d += f"\033[00m{key}: \033[94mtrue\033[00m, "
                    else:
                        d += f"\033[00m{key}: \033[91mfalse\033[00m, "

            if type(key) == int and type(val) == str:
                if len(data.keys()) - num == 0:
                    d += f"\033[93m{key}\033[00m: \033[32m'{val}'\033[00m "
                else:
                    d += f"\033[93m{key}\033[00m: \033[32m'{val}'\033[00m, "

            if type(key) == int and type(val) == int:
                if len(data.keys()) - num == 0:
                    d += f"\033[93m{key}\033[00m: \033[93m{val}\033[00m "
                else:
                    d += f"\033[93m{key}\033[00m: \033[93m{val}\033[00m, "

            if type(key) == int and type(val) == bool:
                if len(data.keys()) - num == 0:
                    if str(val) == "True":
                        d += f"\033[93m{key}\033[00m: \033[94mtrue\033[00m "
                    else:
                        d += f"\033[93m{key}\033[00m: \033[91mfalse\033[00m "
                else:
                    if str(val) == "True":
                        d += f"\033[93m{key}\033[00m: \033[94mtrue\033[00m, "
                    else:
                        d += f"\033[93m{key}\033[00m: \033[91mfalse\033[00m, "

            if type(key) == int and type(val) == dict:
                if len(data.keys()) - num == 0:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.DictParse(val)} "
                else:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.DictParse(val)}, "

            if type(key) == int and type(val) == tuple:
                if len(data.keys()) - num == 0:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.TupleParse(val)} "
                else:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.TupleParse(val)}, "

            if type(key) == int and type(val) == set:
                if len(data.keys()) - num == 0:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.SetParse(val)} "
                else:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.SetParse(val)}, "

            if type(key) == int and type(val) == list:
                if len(data.keys()) - num == 0:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.ListParse(val)} "
                else:
                    d += f"\033[93m{key}\033[00m: \033[00m{self.ListParse(val)}, "

            if type(key) == str and type(val) == dict:
                if len(data.keys()) - num == 0:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.DictParse(val)} "
                else:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.DictParse(val)}, "

            if type(key) == str and type(val) == tuple:
                if len(data.keys()) - num == 0:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.TupleParse(val)} "
                else:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.TupleParse(val)}, "

            if type(key) == str and type(val) == set:
                if len(data.keys()) - num == 0:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.SetParse(val)} "
                else:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.SetParse(val)}, "

            if type(key) == str and type(val) == list:
                if len(data.keys()) - num == 0:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.ListParse(val)} "
                else:
                    d += f"\033[00m{key}\033[00m: \033[00m{self.ListParse(val)}, "

        d += "\033[00m}"

        return d

    def ListParseFlex(self, data: list, flex: int = 2):
        dy = str(json.dumps(data, indent=flex))
        
        for item in data:
            if type(item) == int or type(item) == float:
                dy = dy.replace(f"1", f"<YELLOW>1<WHITE>").replace(f"2", f"<YELLOW>2<WHITE>").replace(f"3", f"<YELLOW>3<WHITE>").replace(f"4", f"<YELLOW>4<WHITE>").replace(f"5", f"<YELLOW>5<WHITE>").replace(f"6", f"<YELLOW>6<WHITE>").replace(f"7", f"<YELLOW>7<WHITE>").replace(f"8", f"<YELLOW>8<WHITE>").replace(f"9", f"<YELLOW>9<WHITE>").replace(f"0", f"<YELLOW>0<WHITE>")

            if type(item) == str:
                dy = dy.replace(f"'{item}'", f"<GREEN>'{item}'<WHITE>").replace(f'"{item}"', f"<GREEN>'{item}'<WHITE>")
            
            if type(item) == bool:
                dy = dy.replace("true", "<BLUE>true<WHITE>").replace("false", "<RED>false<WHITE>")

            if type(item) == list:
                dy = dy.replace(item, self.ListParseFlex(item, flex=flex))

        return "\033[00m" + dy.replace("<BLUE>", "\033[94m").replace("<RED>", "\033[91m").replace("<GREEN>", "\033[32m").replace("<YELLOW>", "\033[93m").replace("<WHITE>", "\033[00m")
    
    def JsonParse(self, data: dict, flex: int = 2):
        ready = json.dumps(data, indent=flex)
        ready = re.sub(r'\b(\d+)\b', r'\033[1;33m\1\033[m', ready)
        ready = re.sub(r'"(.*?)"', r"\033[32m'\033[1;32m\1'\033[m", ready)
        for _ in data.keys():
            ready = ready.replace(f"'{str(_)}'", f"\033[00m{str(_)}")
            ready = ready.replace("false", "\033[91mfalse\033[00m")
            ready = ready.replace("true", "\033[94mtrue\033[00m")
        
        ready = ready.replace("'", "\033[32m'").replace(",", "\033[00m,").replace(":", "\033[00m:")
        return ready
    
    def JsonRichParse(self, data: dict):
        rich.print(data)


from .network import QiNetwork
import random

class QiUpdater(object):
    def __init__(self, Auth, Key, UpdateResult, UseFakeUserAgent: bool = True, Proxy = None) -> None:
        self.auth = str(Auth)
        self.key = str(Key)
        self.UpResult = UpdateResult
        self.ufa = UseFakeUserAgent
        self.proxy = Proxy
        self.networkClient = QiNetwork(self.auth, self.key, self.proxy)
    
    @property
    def status(self) -> str:
        return self.UpResult['status'] if "status" in self.UpResult.keys() else "Null"

    @property
    def status_det(self) -> str:
        return self.UpResult['status_det']if "status_det" in self.UpResult.keys() else "Null"
    
    @property
    def data(self) -> dict:
        return self.UpResult['data'] if "data" in self.UpResult.keys() else "Null"
    
    @property
    def chats(self) -> list:
        return self.data['chats'] if type(self.data) == dict and "chats" in self.data.keys() else "Null"
    
    @property
    def getLastChat(self) -> dict:
        return self.chats[0] if type(self.data) == dict and "chats" in self.data.keys() else "Null"
    
    @property
    def lastChat(self) -> dict:
        return self.chats[0] if type(self.data) == dict and "chats" in self.data.keys() else "Null"

    @property
    def newState(self) -> int:
        return self.data['new_state'] if type(self.data) == dict and 'new_state' in self.data.keys() else "Null"
    
    @property
    def timeStamp(self) -> str:
        return self.data['time_stamp'] if type(self.data) == dict and 'time_stamp' in self.data.keys() else "Null"
    
    @property
    def objectGuid(self) -> str:
        return self.getLastChat['object_guid'] if type(self.getLastChat) == dict and 'object_guid' in self.getLastChat.keys() else "Null"
    
    @property
    def chatId(self) -> str:
        return self.getLastChat['object_guid'] if type(self.getLastChat) == dict and 'object_guid' in self.getLastChat.keys() else "Null"
    
    @property
    def access(self) -> list:
        return self.getLastChat['access'] if type(self.getLastChat) == dict and 'access' in self.getLastChat.keys() else "Null"
    
    @property
    def countUnseen(self) -> int:
        return self.getLastChat['count_unseen'] if type(self.getLastChat) == dict and 'count_unseen' in self.getLastChat.keys() else "Null"
    
    @property
    def isMute(self) -> bool:
        return self.getLastChat['is_mute'] if type(self.getLastChat) == dict and 'is_mute' in self.getLastChat.keys() else "Null"
    
    @property
    def isPinned(self) -> bool:
        return self.getLastChat['is_pinned'] if type(self.getLastChat) == dict and 'is_pinned' in self.getLastChat.keys() else "Null"
    
    @property
    def getLastMessage(self) -> dict:
        return self.getLastChat['last_message'] if type(self.getLastChat) == dict and 'last_message' in self.getLastChat.keys() else "Null"
    
    @property
    def lastMessage(self) -> dict:
        return self.getLastChat['last_message'] if type(self.getLastChat) == dict and 'last_message' in self.getLastChat.keys() else "Null"
    
    @property
    def lastMessageId(self) -> str:
        return self.getLastMessage['message_id'] if type(self.getLastMessage) == dict and 'message_id' in self.getLastMessage.keys() else "Null"
    
    @property
    def messageId(self) -> str:
        return self.getLastMessage['message_id'] if type(self.getLastMessage) == dict and 'message_id' in self.getLastMessage.keys() else "Null"
    
    @property
    def lastMessageType(self) -> str:
        return self.getLastMessage['message_id'] if type(self.getLastMessage) == dict and 'message_id' in self.getLastMessage.keys() else "Null"
    
    @property
    def text(self) -> str:
        return self.getLastMessage['text'] if type(self.getLastMessage) == dict and 'text' in self.getLastMessage.keys() else "Null"
    
    @property
    def authorObjectGuid(self) -> str:
        return self.getLastMessage['author_object_guid'] if type(self.getLastMessage) == dict and 'author_object_guid' in self.getLastMessage.keys() else "Null"
    
    @property
    def isMine(self) -> str:
        return self.getLastMessage['is_mine'] if type(self.getLastMessage) == dict and 'is_mine' in self.getLastMessage.keys() else "Null"
    
    @property
    def authorTitle(self) -> str:
        return self.getLastMessage['author_title'] if type(self.getLastMessage) == dict and 'author_title' in self.getLastMessage.keys() else "Null"
    
    @property
    def authorType(self) -> str:
        return self.getLastMessage['author_type'] if type(self.getLastMessage) == dict and 'author_type' in self.getLastMessage.keys() else "Null"
    
    @property
    def lastSeenMyMid(self) -> str:
        return self.getLastChat['last_seen_my_mid'] if type(self.getLastChat) == dict and 'last_seen_my_mid' in self.getLastChat.keys() else "Null"
    
    @property
    def lastSeenPeerMid(self) -> str:
        return self.getLastChat['last_seen_peer_mid'] if type(self.getLastChat) == dict and 'last_seen_peer_mid' in self.getLastChat.keys() else "Null"
    
    @property
    def lastChatStatus(self) -> str:
        return self.getLastChat['status'] if type(self.getLastChat) == dict and 'status' in self.getLastChat.keys() else "Null"
    
    @property
    def time(self) -> str:
        return self.getLastChat['time'] if type(self.getLastChat) == dict and 'time' in self.getLastChat.keys() else "Null"
    
    @property
    def pinnedMessageId(self) -> str:
        return self.getLastChat['pinned_message_id'] if type(self.getLastChat) == dict and 'pinned_message_id' in self.getLastChat.keys() else "Null"
    
    @property
    def absObjects(self) -> dict:
        return self.getLastChat['abs_object'] if type(self.getLastChat) == dict and 'abs_object' in self.getLastChat.keys() else "Null"
    
    @property
    def absObjectGuid(self) -> str:
        return self.absObjects['object_guid'] if type(self.absObjects) == dict and 'object_guid' in self.absObjects.keys() else "Null"
    
    @property
    def absType(self) -> str:
        return self.absObjects['type'] if type(self.absObjects) == dict and 'type' in self.absObjects.keys() else "Null"
    
    @property
    def absTitle(self) -> str:
        return self.absObjects['title'] if type(self.absObjects) == dict and 'title' in self.absObjects.keys() else "Null"
    
    @property
    def absAvatarThumbnail(self) -> dict:
        return self.absObjects['avatar_thumbnail'] if type(self.absObjects) == dict and 'avatar_thumbnail' in self.absObjects.keys() else "Null"
    
    @property
    def absAvatarFileId(self) -> str:
        return self.absAvatarThumbnail['file_id'] if type(self.absAvatarThumbnail) == dict and 'file_id' in self.absAvatarThumbnail.keys() else "Null"
    
    @property
    def absAvatarMime(self) -> str:
        return self.absAvatarThumbnail['mime'] if type(self.absAvatarThumbnail) == dict and 'mime' in self.absAvatarThumbnail.keys() else "Null"
    
    @property
    def absAvatarDcId(self) -> str:
        return self.absAvatarThumbnail['dc_id'] if type(self.absAvatarThumbnail) == dict and 'dc_id' in self.absAvatarThumbnail.keys() else "Null"
    
    @property
    def absAvatarAccessHashRec(self) -> str:
        return self.absAvatarThumbnail['access_hash_rec'] if type(self.absAvatarThumbnail) == dict and 'access_hash_rec' in self.absAvatarThumbnail.keys() else "Null"
    
    @property
    def isVerified(self) -> bool:
        return self.absObjects['is_verified'] if type(self.absObjects) == dict and 'is_verified' in self.absObjects.keys() else "Null"
    
    @property
    def isDeleted(self) -> bool:
        return self.absObjects['is_deleted'] if type(self.absObjects) == dict and 'is_deleted' in self.absObjects.keys() else "Null"
    
    @property
    def isBlocked(self) -> bool:
        return self.getLastChat['is_blocked'] if type(self.getLastChat) == dict and 'is_blocked' in self.getLastChat.keys() else "Null"
    
    @property
    def lastDeletedMid(self) -> str:
        return self.getLastChat['last_deleted_mid'] if type(self.getLastChat) == dict and 'last_deleted_mid' in self.getLastChat.keys() else "Null"
    
    @property
    def groupMyLastSendTime(self) -> int:
        return self.getLastChat['group_my_last_send_time'] if type(self.getLastChat) == dict and 'group_my_last_send_time' in self.getLastChat.keys() else "Null"
    
    @property
    def pinnedMessageIds(self) -> list:
        return self.getLastChat['pinned_last_message_ids'] if type(self.getLastChat) == dict and 'pinned_last_message_ids' in self.getLastChat.keys() else "Null"
    
    def replyTo(self, text: str) -> dict:
        """
        Send a Message and reply on that
        """
        return self.networkClient.option({"object_guid": self.objectGuid, "text": text, "rnd": random.random() * 1e6 + 1, "reply_to_message_id": self.messageId}, "sendMessage", self.ufa)